package michael.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * http://sjsky.iteye.com
 * @author michael
 */
public class CollectionConvertTest {

    /**
     * List <-> Array
     */
    public static void testListArray() {

        System.out.println("==============================");

        // List -> Array
        System.out.println("List -> Array ");
        List<String> srclist = new ArrayList<String>();
        srclist.add("src_list_a");
        srclist.add("src_list_b");

        String[] destArr = srclist.toArray(new String[srclist.size()]);
        System.out.println("List -> Array 集合中元素的顺序不变");
        srclist.clear();
        System.out.println("srclist clear()");
        System.out.println("destArr length=" + destArr.length);
        if (destArr.length == 2) {
            System.out.println("List -> Array for String : 深度copy");
        } else {
            System.out.println("List -> Array for String : 浅度copy");
        }

        List<TestVo> srcVolist = new ArrayList<TestVo>();
        TestVo vo1 = new TestVo("michael");
        srcVolist.add(vo1);
        srcVolist.add(new TestVo("xiaoxiao"));
        srcVolist.add(new TestVo("vo_3"));

        TestVo[] destVoArr = srcVolist.toArray(new TestVo[srcVolist.size()]);
        System.out.println("List -> Array 集合中元素的顺序不变");
        vo1.setName("xiaoxiao_new");
        System.out.println("srcVolist[0]的属性Name修改为：xiaoxiao_new");
        System.out.println("destVoArr[0]的属性Name当前值：" + destVoArr[0].getName());
        if ("xiaoxiao_new".equals(destVoArr[0].getName())) {
            System.out.println("List -> Array for Bean : 浅度copy");
        } else {
            System.out.println("List -> Array for Bean : 深度copy");
        }

        // Array -> List
        System.out.println("Array -> List");
        String[] srcArr = new String[] { "src_arr_1", "src_arr_2" };
        List<String> destList = Arrays.asList(srcArr);
        srcArr[0] = "src_arr_1_new";
        if ("src_arr_1_new".equals(destList.get(0))) {
            System.out.println("Array -> List for String : 深度copy");
        } else {
            System.out.println("Array -> List for String : 浅度copy");
        }

        TestVo vo3 = new TestVo("vo_3");
        TestVo[] srcObjArr = new TestVo[] { vo3, new TestVo("vo_4") };

        List<TestVo> destObjList = Arrays.asList(srcObjArr);
        vo3.setName("vo_3_new");
        System.out.println(" srcObjArr[0]的属性Name修改为：vo_3_new");
        System.out.println("destObjList[0]的属性Name当前值："
                + destObjList.get(0).getName());
        if ("vo_3_new".equals(destObjList.get(0).getName())) {
            System.out.println("Array -> List for Bean : 浅度copy");
        } else {
            System.out.println("Array -> List for Bean : 深度copy");
        }
        System.out.println("==============================");
    }

    /**
     * Set <-> Array
     */
    public static void testSetArray() {
        System.out.println("==============================");
        // Set -> Array
        System.out.println("Set -> Array");
        System.out.println("Set -> Array 集合中元素的顺序会变");
        Set<String> srcSet = new HashSet<String>();
        srcSet.add("michael");
        srcSet.add("xiaoxiao");
        String[] destArr = srcSet.toArray(new String[srcSet.size()]);
        System.out.println(" String destArr size: " + destArr.length);

        Set<TestVo> srcObjSet = new HashSet<TestVo>();
        TestVo vo1 = new TestVo("michael");
        srcObjSet.add(vo1);
        srcObjSet.add(new TestVo("xiaoxiao"));

        TestVo[] destObjArr = srcObjSet.toArray(new TestVo[srcObjSet.size()]);
        vo1.setName("michael_new");
        System.out.println(" Bean destArr size: " + destArr.length);
        boolean deepFlag = true;
        for (TestVo vo : destObjArr) {
            if ("michael_new".equals(vo.getName())) {
                deepFlag = false;
                break;
            }
        }
        if (deepFlag) {
            System.out.println("Set -> Array for Bean : 深度copy");
        } else {
            System.out.println("Set -> Array for Bean : 浅度copy");
        }

        // Array -> Set
        System.out.println("Array -> Set");
        System.out.println("Array -> Set 集合中元素的顺序会变");
        String[] srcArr = new String[] { "michael", "xiaoxiao" };
        Set<String> destSet = new HashSet<String>(Arrays.asList(srcArr));

        System.out.println(" String destSet size: " + destSet.size());

        TestVo vo3 = new TestVo("michael");
        TestVo[] srcObjArr = new TestVo[] { vo1, new TestVo("xiaoxiao") };

        Set<TestVo> destObjSet = new HashSet<TestVo>(Arrays.asList(srcObjArr));
        vo3.setName("michael_new");
        System.out.println(" Bean dest Set size: " + destObjSet.size());
        deepFlag = true;
        for (TestVo vo : destObjSet) {
            if ("michael_new".equals(vo.getName())) {
                deepFlag = false;
                break;
            }
        }
        if (deepFlag) {
            System.out.println("Array -> Set for Bean : 深度copy");
        } else {
            System.out.println("Array -> Set for Bean : 浅度copy");
        }

        System.out.println("==============================");
    }

    /**
     * List <-> Set
     */
    public static void testListSet() {
        System.out.println("==============================");
        // List -> Set
        System.out.println("List -> Set");
        System.out.println("List -> Set 集合中元素的顺序不变");
        List<String> srclist = new ArrayList<String>();
        srclist.add("src_list_a");
        srclist.add("src_list_b");

        Set<String> destSet = new HashSet<String>(srclist);
        srclist.clear();
        if (destSet.size() == 2) {
            System.out.println("List -> Set for String : 深度copy");
        } else {
            System.out.println("List -> Set for String : 浅度copy");
        }

        List<TestVo> srcObjList = new ArrayList<TestVo>();
        TestVo vo1 = new TestVo("michael");
        srcObjList.add(vo1);
        srcObjList.add(new TestVo("xiaoxiao"));

        Set<TestVo> destObjectSet = new HashSet<TestVo>(srcObjList);
        vo1.setName("michael_new");

        if ("michael_new".equals(destObjectSet.iterator().next().getName())) {
            System.out.println("List -> Set for Bean : 浅度copy");
        } else {
            System.out.println("List -> Set for Bean : 深度copy");
        }

        // Set -> List
        System.out.println("Set -> List");
        System.out.println("List -> Set 集合中元素的顺序会变");
        Set<String> srcSet = new HashSet<String>();
        srcSet.add("src_set_a");
        srcSet.add("src_set_b");
        List<String> destList = new ArrayList<String>(srcSet);
        srcSet.clear();
        if (destList.size() == 2) {
            System.out.println("Set -> List for String : 深度copy");
        } else {
            System.out.println("Set -> List for String : 浅度copy");
        }

        Set<TestVo> srcObjSet = new HashSet<TestVo>();
        TestVo vo3 = new TestVo("vo_3");
        srcObjSet.add(vo3);
        srcObjSet.add(new TestVo("vo_4"));

        List<TestVo> destObjList = new ArrayList<TestVo>(srcObjSet);
        vo3.setName("vo_3_new");
        if ("vo_3_new".equals(destObjList.get(0).getName())
                || "vo_3_new".equals(destObjList.get(1).getName())) {
            System.out.println("Set -> List for Bean : 浅度copy");
        } else {
            System.out.println("Set -> List for Bean : 深度copy");
        }
        System.out.println("==============================");
    }

    /**
     * List <-> Collection
     */
    public static void testListCollection() {

        System.out.println("==============================");
        System.out.println("List -> Collection");
        List<String> list = new ArrayList<String>();
        list.add("michael");
        list.add("xiaoxiao");
        Collection<String> destCol = list;
        System.out.println("destCol size:" + destCol.size());
        System.out.println("List继承于Collection，可以直接赋值");

        System.out.println("Collection -> List");
        Collection<String> col = new ArrayList<String>();
        col.add("michael");
        col.add("xiaoxiao");
        List<String> destList = new ArrayList<String>(col);
        System.out.println("destList size:" + destList.size());

        System.out.println("==============================");
    }

    /**
     * ArrayCopy
     */
    public static void testArrayCopy() {
        System.out.println("==============================");
        System.out.println("Array Copy :String");
        String[] srcArr = new String[] { "michael", "xiaoxiao" };
        String[] destArr = new String[srcArr.length];

        System.arraycopy(srcArr, 0, destArr, 0, srcArr.length);

        System.out.println("Array Copy : Bean");

        TestVo vo1 = new TestVo("michael");
        TestVo[] srcObjArr = new TestVo[] { vo1, new TestVo("xiaoxiao") };
        TestVo[] destObjArr = new TestVo[srcArr.length];

        System.arraycopy(srcObjArr, 0, destObjArr, 0, srcObjArr.length);
        vo1.setName("michael_new");
        System.out.println("srcObjArr[0]的属性name修改为：michael_new");
        System.out
                .println("destObjArr[0]的属性name当前值：" + destObjArr[0].getName());
        if (srcObjArr[0] == destObjArr[0]) {
            System.out.println("Array Copy for Bean : 浅度copy");
        } else {
            System.out.println("Array Copy for Bean : 深度copy");
        }
        System.out.println("==============================");
    }

    /**
     * List copy
     */
    public static void testListCopy() {
        System.out.println("==============================");
        System.out.println("List Copy :String");

        List<String> srcList = new ArrayList<String>();
        srcList.add("michael");
        srcList.add("xiaoxiao");

        List<String> descList = srcList;

        List<String> descList2 = new ArrayList<String>(srcList);

        List<String> descList3 = new ArrayList<String>(Arrays
                .asList(new String[srcList.size()]));
        Collections.copy(descList3, srcList);

        srcList.clear();
        System.out.println("源List清空");
        System.out.println("直接赋值descList = srcList 的方法copy后的List     size: "
                + descList.size());
        System.out.println("new ArrayList<String>(srcList)的方法copy后的List size: "
                + descList2.size());
        System.out.println("Collections.copy(dest,src)的方法copy后的List     size: "
                + descList3.size());

        System.out.println("==============================");
    }

    /**
     * @param args
     */
    public static void main(String[] args) {

        CollectionConvertTest.testListArray();

        CollectionConvertTest.testSetArray();

        CollectionConvertTest.testListSet();

        CollectionConvertTest.testListCollection();

        CollectionConvertTest.testArrayCopy();

        CollectionConvertTest.testListCopy();

    }

    static class TestVo {
        private String name;

        /**
         * @param name
         */
        public TestVo(String name) {
            this.name = name;
        }

        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param pName the name to set
         */
        public void setName(String pName) {
            name = pName;
        }

    }
}
